// Compact Disc Detect & Execute
//
// Copyright(C) 2002-03 Eric Lathrop <eric@ericlathrop.com>
//

#include "list.h"
#include <stdlib.h>
#include <stdio.h>

// push data onto the top of the list
//
// returns: the new list
//
list * list_push(list * l, void * data)
{
	list * temp = malloc(sizeof(list));
	if (temp == NULL)
	{
		fprintf(stderr, "Error: list_push(): couldn't allocate memory!\n");
		exit(1);
	}
	temp->data = data;
	temp->next = l;
	return temp;
}

// pop the top item off the list,
// store a pointer to its data in data
// and free() the item
//
// returns: the new list
//
list * list_pop(list * l, void ** data)
{
	list * temp = NULL;
	if (l == NULL)
	{
		*data = NULL;
		return NULL;
	}
	temp = l->next;
	*data = l->data;
	l->data = NULL;
	l->next = NULL;
	free(l);
	return temp;
}

// returns the nth item of the list
// store a pointer to its data in data
// and free the item
//
// returns: the new list
//
list * list_removenth(list * l, void ** data, int n)
{
	int i=0;
	list * temp;
	list * holder;
	if (l == NULL)
	{
		*data = NULL;
		return NULL;
	}
	if (n < 0)
	{
		*data = NULL;
		return l;
	}
	
	if (n == 0)
	{
		temp = l->next;
		*data = l->data;
		l->next = NULL;
		l->data = NULL;
		free(l);
		return temp;
	}
	temp = l;
	for (i=0; i<n-1; i++)
	{
		temp = temp->next;
		if (temp == NULL)
		{
			*data = NULL;
			return l;
		}
	}
	holder = temp->next;
	temp->next = temp->next->next;
	*data = holder->data;
	holder->next = NULL;
	holder->data = NULL;
	free(holder);
	return l;
}

// get data from nth list element
void * list_nth(list * l, int n)
{
	int i;
	if (n < 0) return NULL;
	for (i=0; i<n; i++)
	{
		if (l == NULL) return NULL;
		l = l->next;
	}
	return l->data;
}

// get size of list
int list_size(list * l)
{
	int i=0;
	while (l != NULL)
	{
		l = l->next;
		i++;
	}
	return i;
}
